@@ -32,17 +32,13 @@ import gobject
 import dbus, dbus.service, dbus.mainloop.glib
 
 import eds
-from configuration import GconfStore, runtime
+from configuration import GconfStore, runtime, dialogs
 
 import stuff
 from KeyBinder import *
 from hamsterdbus import HAMSTER_URI, HamsterDbusController
 
 # controllers for other windows
-from edit_activity import CustomFactController
-from stats import StatsViewer
-from about import show_about
-from preferences import PreferencesEditor
 import widgets
 import idle
 
@@ -114,6 +110,10 @@ class PanelButton(gtk.ToggleButton):
         self.set_property('active', is_active)
 
     def set_text(self, activity, duration):
+        activity = stuff.escape_pango(activity)
+        if len(activity) > 25:  #ellipsize at some random length
+            activity = "%s%s" % (activity[:25], "&#8230;")
+
         self.activity = activity
         self.duration = duration
         self.reformat_label()
@@ -126,7 +126,7 @@ class PanelButton(gtk.ToggleButton):
             else:
                 label = "%s %s" % (self.activity, self.duration)
         
-        label = '<span gravity="south">%s</span>' % stuff.escape_pango(label)
+        label = '<span gravity="south">%s</span>' % label
         self.label.set_markup("") #clear - seems to fix the warning
         self.label.set_markup(label)
 
@@ -230,6 +230,7 @@ class HamsterApplet(object):
         self.window = self._gui.get_object('hamster-window')
 
         self.new_name = widgets.ActivityEntry()
+        self.new_name.connect("value-entered", self.on_switch_activity_clicked)
         widgets.add_hint(self.new_name, _("Time and Name"))
         self.get_widget("new_name_box").add(self.new_name)
         self.new_name.connect("changed", self.on_activity_text_changed)
@@ -325,40 +326,7 @@ class HamsterApplet(object):
             label = "%s" % _(u"No activity")
             self.button.set_text(label, None)
             
-        self.set_last_activity()
 
-        
-    def set_last_activity(self):
-        activity = runtime.storage.get_last_activity()
-        self.get_widget("stop_tracking").set_sensitive(activity != None)
-        
-        
-        if activity:
-            self.get_widget("switch_activity").show()
-            self.get_widget("start_tracking").hide()
-            
-            delta = dt.datetime.now() - activity['start_time']
-            duration = delta.seconds /  60
-            
-            self.get_widget("last_activity_duration").set_text(stuff.format_duration(duration) or _("Just started"))
-            self.get_widget("last_activity_name").set_text(activity['name'])
-            if activity['category'] != _("Unsorted"):
-                self.get_widget("last_activity_category") \
-                    .set_text(" - %s" % activity['category'])
-
-            self.get_widget("last_activity_description").set_text(activity['description'] or "")
-
-            self.tag_box.draw(activity["tags"])
-        else:
-            self.get_widget("switch_activity").hide()
-            self.get_widget("start_tracking").show()
-
-            self.get_widget("last_activity_name").set_text(_("No activity"))
-            self.get_widget("last_activity_duration").set_text("")
-            self.get_widget("last_activity_category").set_text("")
-            self.tag_box.draw([])
-            self.get_widget("last_activity_description").set_text("")
-         
     def check_user(self):
         if not self.notify_interval: #no interval means "never"
             return
@@ -378,8 +346,7 @@ class HamsterApplet(object):
                 self.notify.msg_low(_(u"No activity"))
 
     def edit_cb(self, n, action):
-        custom_fact = CustomFactController(self, None, self.last_activity['id'])
-        custom_fact.show()
+        dialogs.edit.show(self.applet, activity_id = self.last_activity['id'])
 
     def switch_cb(self, n, action):
         self.__show_toggle(None, not self.button.get_active())	
@@ -388,15 +355,17 @@ class HamsterApplet(object):
     def load_day(self):
         """sets up today's tree and fills it with records
            returns information about last activity"""
-        #today is 5.5 hours ago because our midnight shift happens 5:30am
+
         today = (dt.datetime.now() - dt.timedelta(hours = self.day_start.hour,
                                                   minutes = self.day_start.minute)).date()
 
-        self.last_activity = runtime.storage.get_last_activity()
-
         self.treeview.clear()
 
         facts = runtime.storage.get_facts(today)
+        if facts and facts[-1]["end_time"] == None:
+            self.last_activity = facts[-1]
+        else:
+            self.last_activity = None
         
         if len(facts) > 10:
             self._gui.get_object("today_box").set_size_request(-1, 250)
@@ -431,7 +400,40 @@ class HamsterApplet(object):
 
             total_string = ", ".join(total_strings)
             self._gui.get_object("fact_totals").set_text(total_string)
-   
+            
+        self.set_last_activity()
+
+    def set_last_activity(self):
+        activity = self.last_activity
+        #sets all the labels and everything as necessary
+        self.get_widget("stop_tracking").set_sensitive(activity != None)
+        
+        
+        if activity:
+            self.get_widget("switch_activity").show()
+            self.get_widget("start_tracking").hide()
+            
+            delta = dt.datetime.now() - activity['start_time']
+            duration = delta.seconds /  60
+            
+            self.get_widget("last_activity_duration").set_text(stuff.format_duration(duration) or _("Just started"))
+            self.get_widget("last_activity_name").set_text(activity['name'])
+            if activity['category'] != _("Unsorted"):
+                self.get_widget("last_activity_category") \
+                    .set_text(" - %s" % activity['category'])
+
+            self.get_widget("last_activity_description").set_text(activity['description'] or "")
+
+            self.tag_box.draw(activity["tags"])
+        else:
+            self.get_widget("switch_activity").hide()
+            self.get_widget("start_tracking").show()
+
+            self.get_widget("last_activity_name").set_text(_("No activity"))
+            self.get_widget("last_activity_duration").set_text("")
+            self.get_widget("last_activity_category").set_text("")
+            self.tag_box.draw([])
+            self.get_widget("last_activity_description").set_text("")
 
     def delete_selected(self):
         selection = self.treeview.get_selection()
@@ -503,12 +505,10 @@ class HamsterApplet(object):
         
 
     """events"""
-
-        
     def on_toggle(self, widget):
         self.__show_toggle(None, self.button.get_active())
 
-    """listview events"""
+
     def on_todays_keys(self, tree, event):
         if (event.keyval == gtk.keysyms.Delete):
             self.delete_selected()
@@ -518,8 +518,7 @@ class HamsterApplet(object):
     
     def _open_edit_activity(self, row, fact):
         """opens activity editor for selected row"""
-        custom_fact = CustomFactController(self, None, fact["id"])
-        custom_fact.show()
+        dialogs.edit.show(self.applet, fact_id = fact["id"])
         
     def on_today_row_activated(self, tree, path, column):
         selection = tree.get_selection()
@@ -552,30 +551,21 @@ class HamsterApplet(object):
     """button events"""
     def on_overview(self, menu_item):
         runtime.dispatcher.dispatch('panel_visible', False)
-        stats_viewer = StatsViewer(self)
-        stats_viewer.show()
+        dialogs.overview.show(self.applet)
 
     def show_overview(self, menu_item, verb):
         return self.on_overview(menu_item)
 
     def on_custom_fact(self, menu_item):
-        custom_fact = CustomFactController(self)
-        custom_fact.show()
+        dialogs.edit.show(self.applet)
 
     def on_about (self, component, verb):
-        if self.applet.about:
-            self.applet.about.present()
-        else:
-            show_about(self.applet)
+        dialogs.about.show()
 
     def show_preferences(self, menu_item, verb):
         runtime.dispatcher.dispatch('panel_visible', False)
-        
-        if self.preferences_editor and self.preferences_editor.window:
-            self.preferences_editor.window.present()
-        else:
-            self.preferences_editor = PreferencesEditor(self)
-            self.preferences_editor.show()
+        dialogs.prefs.show(self.applet)
+
     
     """signals"""
     def after_activity_update(self, widget, renames):
@@ -592,7 +582,7 @@ class HamsterApplet(object):
         # state values: 0 = active, 1 = idle
 
         # refresh when we are out of idle
-        # (like, instantly after computer has been turned on!
+        # for example, instantly after coming back from suspend
         if state == 0:
             self.refresh_hamster()
         elif self.timeout_enabled and self.last_activity and \
@@ -604,17 +594,16 @@ class HamsterApplet(object):
     def on_day_start_changed(self, event, new_minutes):
         self.day_start = self.config.get_day_start()
         self.load_day()
+        self.update_label()
 
     """global shortcuts"""
     def on_keybinding_activated(self, event, data):
         self.__show_toggle(None, not self.button.get_active())
         
     def on_timeout_enabled_changed(self, event, enabled):
-        # if enabled, set to value, otherwise set to zero, which means disable
         self.timeout_enabled = enabled
 
     def on_notify_on_idle_changed(self, event, enabled):
-        # if enabled, set to value, otherwise set to zero, which means disable
         self.notify_on_idle = enabled
 
     def on_notify_interval_changed(self, event, new_interval):
@@ -623,29 +612,6 @@ class HamsterApplet(object):
         else:
             self.notify_interval = None
 
-        
-    def on_more_info_button_clicked(self, button):
-        def on_response(self, widget):
-            self.destroy()
-
-        message_dialog = gtk.MessageDialog(buttons = gtk.BUTTONS_OK)
-        message_dialog.set_property("title", _("What should be typed in the activity box?"))
-        message_dialog.connect("response", on_response)
-        
-        more_info = _("""There is a simple syntax that enables you to add details to your activities:
-        
-"@" symbol marks a category. Example: "watering flowers@home" will start tracking the activity "watering flowers" in the category "home".
-
-Commas (",") mark beginning of a description. Example: "watering flowers, begonias and forgetmenots" will start tracking the activity "watering flowers" and add the description "begonias and forgetmenots" to it.
-
-Both can be combined: "watering flowers@home, begonias and forgetmenots" will work just fine!
-
-Now, start tracking!
-        """)
-        
-        message_dialog.set_markup(more_info)
-        message_dialog.show()
-
     def on_activity_text_changed(self, widget):
         self.get_widget("switch_activity").set_sensitive(widget.get_text() != "")
 
@@ -657,7 +623,7 @@ Now, start tracking!
         runtime.dispatcher.dispatch('panel_visible', False)
 
     def on_stop_tracking_clicked(self, widget):
-        runtime.storage.touch_fact(runtime.storage.get_last_activity())
+        runtime.storage.touch_fact(self.last_activity)
         self.last_activity = None
         runtime.dispatcher.dispatch('panel_visible', False)
 
