@@ -33,8 +33,7 @@ http://projecthamster.wordpress.com/
 
 """
 
-import gtk
-import gobject
+import gtk, gobject
 import cairo, pango
 import copy
 import math
@@ -42,20 +41,10 @@ from sys import maxint
 import datetime as dt
 import time
 import colorsys
+import logging
+
 import graphics
 
-# Tango colors
-light = [(252, 233, 79), (252, 175, 62),  (233, 185, 110),
-         (138, 226, 52), (114, 159, 207), (173, 127, 168), 
-         (239, 41,  41), (238, 238, 236), (136, 138, 133)]
-
-medium = [(237, 212, 0),  (245, 121, 0),   (193, 125, 17),
-          (115, 210, 22), (52,  101, 164), (117, 80,  123), 
-          (204, 0,   0),  (211, 215, 207), (85, 87, 83)]
-
-dark = [(196, 160, 0), (206, 92, 0),    (143, 89, 2),
-        (78, 154, 6),  (32, 74, 135),   (92, 53, 102), 
-        (164, 0, 0),   (186, 189, 182), (46, 52, 54)]
 
 def size_list(set, target_set):
     """turns set lenghts into target set - trim it, stretches it, but
@@ -88,6 +77,15 @@ def get_limits(set, stack_subfactors = True):
     return min_value, max_value
     
 
+class Bar(object):
+    def __init__(self, value, size = 0):
+        self.value = value
+        self.size = size
+    
+    def __repr__(self):
+        return str((self.value, self.size))
+        
+
 class Chart(graphics.Area):
     """Chart constructor. Optional arguments:
         self.max_bar_width     = pixels. Maximal width of bar. If not specified,
@@ -101,7 +99,6 @@ class Chart(graphics.Area):
         self.background        = Tripplet-tuple of background color in RGB
         self.chart_background  = Tripplet-tuple of chart background color in RGB
         self.bar_base_color    = Tripplet-tuple of bar color in RGB
-        self.bars_beveled      = Should bars be beveled. 
 
         self.show_scale        = Should we show scale values. See grid_stride!
         self.grid_stride       = Step of grid. If expressed in normalized range
@@ -118,19 +115,22 @@ class Chart(graphics.Area):
         self.labels_at_end     = If stack bars are displayed, this allows to
                                  show them at right end of graph.
     """
+    __gsignals__ = {
+        "bar-clicked": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, )),
+    }
     def __init__(self, **args):
         graphics.Area.__init__(self)
 
+        # options
         self.max_bar_width     = args.get("max_bar_width", 500)
         self.legend_width      = args.get("legend_width", 0)
-        self.animate           = args.get("animate", True)
+        self.animation           = args.get("animate", True)
 
         self.background        = args.get("background", None)
         self.chart_background  = args.get("chart_background", None)
         self.bar_base_color    = args.get("bar_base_color", None)
 
         self.grid_stride       = args.get("grid_stride", None)
-        self.bars_beveled      = args.get("bars_beveled", False)
         self.values_on_bars    = args.get("values_on_bars", False)
         self.value_format      = args.get("value_format", "%s")
         self.show_scale        = args.get("show_scale", False)
@@ -139,14 +139,47 @@ class Chart(graphics.Area):
         self.labels_at_end     = args.get("labels_at_end", False)
         self.framerate         = args.get("framerate", 60)
 
-        #and some defaults
-        self.current_max = None
-        self.integrators = []
-        self.moving = False
+        self.interactive       = args.get("interactive", False) # if the bars are clickable
+
+        # other stuff
+        self.bars = []
+        self.keys = []
+        self.data = None
+        self.stack_keys = []
         
+        self.key_colors = {} # key:color dictionary. if key's missing will grab basecolor
+        self.stack_key_colors = {} # key:color dictionary. if key's missing will grab basecolor
+        
+
+        # use these to mark area where the "real" drawing is going on
+        self.graph_x, self.graph_y = 0, 0
+        self.graph_width, self.graph_height = None, None
+        
+        self.mouse_bar = None
+        if self.interactive:
+            self.connect("mouse-over", self.on_mouse_over)
+            self.connect("button-release", self.on_clicked)
+            
+        self.bars_selected = []
+        
+    
+    def on_mouse_over(self, area, region):
+        if region:
+            self.mouse_bar = int(region[0])
+        else:
+            self.mouse_bar = None
+            
+        self.redraw_canvas()
+        
+    def on_clicked(self, area, bar):
+        self.emit("bar-clicked", self.mouse_bar)
+    
+    def select_bar(self, index):
+        pass
+
     def get_bar_color(self, index):
         # returns color darkened by it's index
-        # the approach reduces contrast by each step because we tend to differ         
+        # the approach reduces contrast by each step
         base_color = self.bar_base_color or (220, 220, 220)
         
         base_hls = colorsys.rgb_to_hls(*base_color)
@@ -170,65 +203,22 @@ class Chart(graphics.Area):
 
         self.show()
 
-        if not data: #if there is no data, let's just draw blank
+        if not data: #if there is no data, just draw blank
             self.redraw_canvas()
             return
 
 
         min, self.max_value = get_limits(data)
 
-        if not self.current_max:
-            self.current_max = graphics.Integrator(0)
-        self.current_max.target(self.max_value)
-        
         self._update_targets()
-        
-        if self.animate:
-            if not self.moving: #if we are moving, then there is a timeout somewhere already
-                gobject.timeout_add(1000 / self.framerate, self._interpolate)
-        else:
-            def finish_all(integrators):
-                for i in range(len(integrators)):
-                    if isinstance(integrators[i], list):
-                        finish_all(integrators[i])
-                    else:
-                        integrators[i].finish()
-    
-            finish_all(self.integrators)
-            self.current_max.finish()
 
-
-            self.redraw_canvas()
-
-
-    def _interpolate(self):
-        """Internal function to do the math, going from previous set to the
-           new one, and redraw graph"""
-        #this can get called before expose    
-        if not self.window:
-            self.redraw_canvas()
-            return False
-
-        #ok, now we are good!
-        self.current_max.update()
-
-
-        def update_all(integrators):
-            still_moving = False
-            for z in range(len(integrators)):
-                if isinstance(integrators[z], list):
-                    still_moving = update_all(integrators[z]) or still_moving
-                else:
-                    still_moving = integrators[z].update() > 0.0001 or still_moving
-            return still_moving
-
-        self.moving = update_all(self.integrators)
+        if not self.animation:
+            self.tweener.finish()
 
         self.redraw_canvas()
-        
-        return self.moving #return if there is still work to do
 
-    def _render(self):
+
+    def on_expose(self):
         # fill whole area 
         if self.background:
             self.fill_area(0, 0, self.width, self.height, self.background)
@@ -238,32 +228,51 @@ class Chart(graphics.Area):
         # calculates new factors and then updates existing set
         max_value = float(self.max_value) or 1 # avoid division by zero
         
-        self.integrators = size_list(self.integrators, self.data)
+        self.bars = size_list(self.bars, self.data)
 
         #need function to go recursive
-        def retarget(integrators, new_values):
+        def retarget(bars, new_values):
             for i in range(len(new_values)):
                 if isinstance(new_values[i], list):
-                    integrators[i] = retarget(integrators[i], new_values[i])
+                    bars[i] = retarget(bars[i], new_values[i])
                 else:
-                    if isinstance(integrators[i], graphics.Integrator) == False:
-                        integrators[i] = graphics.Integrator(0)
+                    if isinstance(bars[i], Bar) == False:
+                        bars[i] = Bar(new_values[i], 0)
+                    else:
+                        bars[i].value = new_values[i]
+                        self.tweener.killTweensOf(bars[i])
 
-                    integrators[i].target(new_values[i] / max_value)
-            
-            return integrators
+                    self.tweener.addTween(bars[i], size = bars[i].value / float(max_value))
+            return bars
     
-        retarget(self.integrators, self.data)
+        retarget(self.bars, self.data)
+
+
+    def longest_label(self, labels):
+        """returns width of the longest label"""
+        max_extent = 0
+        for label in labels:
+            self.layout.set_text(label)
+            label_w, label_h = self.layout.get_pixel_size()
+            max_extent = max(label_w + 5, max_extent)
+        
+        return max_extent
     
     def draw(self):
-        print "OMG OMG, not implemented!!!"
+        logging.error("OMG OMG, not implemented!!!")
 
 
 class BarChart(Chart):
-    def _render(self):
-        context = self.context
-        Chart._render(self)
-        
+    def on_expose(self):
+        Chart.on_expose(self)
+
+        if not self.data:
+            return
+
+        context = self.context        
+        context.set_line_width(1)
+
+
         # determine graph dimensions
         if self.show_stack_labels:
             legend_width = self.legend_width or self.longest_label(self.keys)
@@ -296,13 +305,6 @@ class BarChart(Chart):
 
         self.context.stroke()
 
-        bar_width = min(self.graph_width / float(len(self.keys)),
-                                                             self.max_bar_width)
-        gap = bar_width * 0.05
-        
-        # flip hamster.graphics matrix so we don't think upside down
-        self.set_value_range(y_max = 0, y_min = self.graph_height)
-
         # bars and keys
         max_bar_size = self.graph_height
         #make sure bars don't hit the ceiling
@@ -313,15 +315,26 @@ class BarChart(Chart):
         prev_label_end = None
         self.layout.set_width(-1)
 
-        for i in range(len(self.keys)):
+        exes = {}
+        x = 0
+        bar_width = min(self.graph_width / float(len(self.keys)), self.max_bar_width)
+        for i, key in enumerate(self.keys):
+            exes[key] = (x + self.graph_x, round(bar_width - 1))
+            
+            x = x + round(bar_width)
+            bar_width = min(self.max_bar_width,
+                            (self.graph_width - x) / float(max(1, len(self.keys) - i - 1)))
+
+
+        for key, bar, data in zip(self.keys, self.bars, self.data):
             self.set_color(graphics.Colors.aluminium[5]);
-            self.layout.set_text(self.keys[i])
+            self.layout.set_text(key)
             label_w, label_h = self.layout.get_pixel_size()
 
-            intended_x = (bar_width * i) + (bar_width - label_w) / 2.0
+            intended_x = exes[key][0] + (exes[key][1] - label_w) / 2
             
             if not prev_label_end or intended_x > prev_label_end:
-                self.move_to(intended_x, -4)
+                self.context.move_to(intended_x, self.graph_height + 4)
                 context.show_layout(self.layout)
             
                 prev_label_end = intended_x + label_w + 3
@@ -329,35 +342,63 @@ class BarChart(Chart):
 
             bar_start = 0
             base_color = self.bar_base_color or (220, 220, 220)
-            bar_x = round(self.graph_x + bar_width * i + gap)
-
+            
             if self.stack_keys:
-                for j in range(len(self.integrators[i])):
-                    factor = self.integrators[i][j].value
-    
-                    if factor > 0:
-                        bar_size = round(max_bar_size * factor)
+                remaining_fractions, remaining_pixels = 1, max_bar_size
+                
+                for j, stack_bar in enumerate(bar):
+                    if stack_bar.size > 0:
+                        bar_size = round(remaining_pixels * (stack_bar.size / remaining_fractions))
+                        remaining_fractions -= stack_bar.size
+                        remaining_pixels -= bar_size
+
                         bar_start += bar_size
                         
-                        self.draw_bar(bar_x,
+                        last_color = self.stack_key_colors.get(self.stack_keys[j]) or self.get_bar_color(j)
+                        self.draw_bar(exes[key][0],
                                       self.graph_height - bar_start,
-                                      round(bar_width - (gap * 2)),
+                                      exes[key][1],
                                       bar_size,
-                                      self.get_bar_color(j))
+                                      last_color)
             else:
-                factor = self.integrators[i].value
-                bar_size = round(max_bar_size * factor)
+                bar_size = round(max_bar_size * bar.size)
                 bar_start = bar_size
 
-                self.draw_bar(bar_x,
+                last_color = self.key_colors.get(key) or base_color
+                self.draw_bar(exes[key][0],
                               self.graph_y + self.graph_height - bar_size,
-                              round(bar_width - (gap * 2)),
+                              exes[key][1],
                               bar_size,
-                              base_color)
+                              last_color)
+
+
+            if self.values_on_bars:  # it is either stack labels or values at the end for now
+                if self.stack_keys:
+                    total_value = sum(data[i])
+                else:
+                    total_value = data[i]
+                
+                self.layout.set_width(-1)
+                self.layout.set_text(self.value_format % total_value)
+                label_w, label_h = self.layout.get_pixel_size()
+    
+
+                if bar_start > label_h + 2:
+                    label_y = self.graph_y + self.graph_height - bar_start + 5
+                else:
+                    label_y = self.graph_y + self.graph_height - bar_start - label_h + 5
+                
+                context.move_to(self.exes[key][0] + (self.exes[key][1] - label_w) / 2.0,
+                                label_y)
+
+                # we are in the bar so make sure that the font color is distinguishable
+                if colorsys.rgb_to_hls(*graphics.Colors.rgb(last_color))[1] < 150:
+                    self.set_color(graphics.Colors.almost_white)
+                else:
+                    self.set_color(graphics.Colors.aluminium[5])        
+
+                context.show_layout(self.layout)
 
-        #fill with white background (necessary for those dragging cases)
-        if self.background:
-            self.fill_area(0, 0, legend_width, self.height, self.background)
 
         #white grid and scale values
         self.layout.set_width(-1)
@@ -370,27 +411,23 @@ class BarChart(Chart):
             
             context.set_line_width(1)
             for i in range(grid_stride, int(self.max_value), grid_stride):
-                y = max_bar_size * (i / self.max_value)
+                y = round(max_bar_size * (i / self.max_value)) + 0.5
 
                 if self.show_scale:
                     self.layout.set_text(self.value_format % i)
                     label_w, label_h = self.layout.get_pixel_size()
                     context.move_to(legend_width - label_w - 8,
-                                    self.get_pixel(y_value=y) - label_h / 2)
+                                    y - label_h / 2)
                     self.set_color(graphics.Colors.aluminium[4])
                     context.show_layout(self.layout)
 
-                self.set_color((255, 255, 255))
-                self.context.move_to(legend_width, self.get_pixel(y_value=y))
-                self.context.line_to(self.width, self.get_pixel(y_value=y))
+                self.set_color("#ffffff")
+                self.context.move_to(legend_width, y)
+                self.context.line_to(self.width, y)
 
 
         #stack keys
-        context.save()
         if self.show_stack_labels:
-            context.set_line_width(1)
-            context.set_antialias(cairo.ANTIALIAS_DEFAULT)
-
             #put series keys
             self.set_color(graphics.Colors.aluminium[5]);
             
@@ -399,9 +436,12 @@ class BarChart(Chart):
 
             # if labels are at end, then we need show them for the last bar! 
             if self.labels_at_end:
-                factors = self.integrators[0]
+                factors = self.bars[-1]
             else:
-                factors = self.integrators[-1]
+                factors = self.bars[0]
+            
+            if isinstance(factors, Bar):
+                factors = [factors]
 
             self.layout.set_ellipsize(pango.ELLIPSIZE_END)
             self.layout.set_width(self.graph_x * pango.SCALE)
@@ -411,12 +451,12 @@ class BarChart(Chart):
                 self.layout.set_alignment(pango.ALIGN_RIGHT)
     
             for j in range(len(factors)):
-                factor = factors[j].value
+                factor = factors[j].size
                 bar_size = factor * max_bar_size
                 
-                if round(bar_size) > 0:
+                if round(bar_size) > 0 and self.stack_keys:
                     label = "%s" % self.stack_keys[j]
-                    
+
                     
                     self.layout.set_text(label)
                     label_w, label_h = self.layout.get_pixel_size()
@@ -447,13 +487,16 @@ class BarChart(Chart):
                         context.line_to(line_x2, round(y + bar_size / 2))
 
         context.stroke()
-        context.restore()
 
 
 class HorizontalBarChart(Chart):
-    def _render(self):
+    def on_expose(self):
+        Chart.on_expose(self)
+
+        if not self.data:
+            return
+
         context = self.context
-        Chart._render(self)
         rowcount, keys = len(self.keys), self.keys
         
         # push graph to the right, so it doesn't overlap
@@ -470,69 +513,87 @@ class HorizontalBarChart(Chart):
             self.fill_area(self.graph_x, self.graph_y, self.graph_width, self.graph_height, self.chart_background)
 
     
-        if not self.data:  #if we have nothing, let's go home
+        if not self.data:  # go home if we have nothing
             return
 
-        
-        bar_width = int(self.graph_height / float(rowcount))
-        bar_width = min(bar_width, self.max_bar_width)
+        positions = {}
+        y = 0
+        bar_width = min(self.graph_height / float(len(self.keys)), self.max_bar_width)
+        for i, key in enumerate(self.keys):
+            positions[key] = (y + self.graph_y, round(bar_width - 1))
+            
+            y = y + round(bar_width)
+            bar_width = min(self.max_bar_width,
+                            (self.graph_height - y) / float(max(1, len(self.keys) - i - 1)))
+
 
-        
         max_bar_size = self.graph_width - 15
-        gap = bar_width * 0.05
-
 
         self.layout.set_alignment(pango.ALIGN_RIGHT)
         self.layout.set_ellipsize(pango.ELLIPSIZE_END)
         
 
         
-        context.set_line_width(0)
+        context.set_line_width(1)
 
         # bars and labels
         self.layout.set_width(legend_width * pango.SCALE)
         
 
         for i, label in enumerate(keys):
+            if self.interactive:
+                self.register_mouse_region(0,
+                                           positions[label][0],
+                                           self.width,
+                                           positions[label][0] + positions[label][1],
+                                           str(i))
+
             self.layout.set_width(legend_width * pango.SCALE)
-            self.set_color(graphics.Colors.aluminium[5])        
-            
             self.layout.set_text(label)
             label_w, label_h = self.layout.get_pixel_size()
-
-            context.move_to(0, (bar_width * i) + (bar_width - label_h) / 2)
+            
+            self.set_color(graphics.Colors.aluminium[5])        
+            context.move_to(0, positions[label][0] + (positions[label][1] - label_h) / 2)
             context.show_layout(self.layout)
 
             base_color = self.bar_base_color or (220, 220, 220)
 
-            gap = bar_width * 0.05
-
-            bar_y = round(self.graph_y + (bar_width * i) + gap)
-
             last_color = (255,255,255)
 
             if self.stack_keys:
                 bar_start = 0
-                for j, factor in enumerate(self.integrators[i]):
-                    if factor.value > 0:
-                        bar_size = round(max_bar_size * factor.value)
-                        bar_height = round(bar_width - (gap * 2))
-                        
-                        last_color = self.get_bar_color(j)
+
+                remaining_fractions, remaining_pixels = 1, max_bar_size
+                
+                for j, stack_bar in enumerate(self.bars[i]):
+                    if stack_bar.size > 0:
+                        bar_size = round(remaining_pixels * (stack_bar.size / remaining_fractions))
+                        remaining_fractions -= stack_bar.size
+                        remaining_pixels -= bar_size
+
+                        last_color = self.stack_key_colors.get(self.stack_keys[j]) or self.get_bar_color(j)
                         self.draw_bar(self.graph_x + bar_start,
-                                      bar_y,
+                                      positions[label][0],
                                       bar_size,
-                                      bar_height,
+                                      positions[label][1],
                                       last_color)
                         bar_start += bar_size
             else:
-                factor = self.integrators[i].value
-                bar_size = round(max_bar_size * factor)
+                bar_size = round(max_bar_size * self.bars[i].size)
                 bar_start = bar_size
 
-                bar_height = round(bar_width - (gap * 2))
-                self.draw_bar(self.graph_x, bar_y, bar_size, bar_height,
-                                                                     base_color)
+                if i in self.bars_selected:
+                    last_color = self.get_style().bg[gtk.STATE_SELECTED].to_string()
+                elif i == self.mouse_bar:
+                    last_color = self.get_style().bg[gtk.STATE_PRELIGHT].to_string()
+                else:
+                    last_color = self.key_colors.get(self.keys[i]) or base_color
+
+                self.draw_bar(self.graph_x,
+                              positions[label][0],
+                              bar_size,
+                              positions[label][1],
+                              last_color)
 
             # values on bars
             if self.stack_keys:
@@ -544,22 +605,23 @@ class HorizontalBarChart(Chart):
             self.layout.set_text(self.value_format % total_value)
             label_w, label_h = self.layout.get_pixel_size()
 
-            vertical_padding = (bar_width - (bar_width + label_h) / 2.0 ) / 2.0
+            vertical_padding = max((positions[label][1] - label_h) / 2.0, 1)
             if  bar_start - vertical_padding < label_w:
                 label_x = self.graph_x + bar_start + vertical_padding
                 self.set_color(graphics.Colors.aluminium[5])        
             else:
-                # we are in the bar so make sure that the font color is distinguishable
-                # this is a hamster fix
-                # TODO - drop the library bit, we will never be adopted
-                if colorsys.rgb_to_hls(*last_color)[1] < 150:
-                    self.set_color(graphics.Colors.almost_white)
+                if i in self.bars_selected:
+                    self.set_color(self.get_style().fg[gtk.STATE_SELECTED].to_string())
                 else:
-                    self.set_color(graphics.Colors.aluminium[5])        
+                    # we are in the bar so make sure that the font color is distinguishable
+                    if colorsys.rgb_to_hls(*graphics.Colors.rgb(last_color))[1] < 150:
+                        self.set_color(graphics.Colors.almost_white)
+                    else:
+                        self.set_color(graphics.Colors.aluminium[5])        
                     
                 label_x = self.graph_x + bar_start - label_w - vertical_padding
             
-            context.move_to(label_x, self.graph_y + (bar_width * i) + (bar_width - label_h) / 2.0)
+            context.move_to(label_x, positions[label][0] + (positions[label][1] - label_h) / 2.0)
             context.show_layout(self.layout)
 
         context.stroke()
@@ -570,15 +632,20 @@ class HorizontalBarChart(Chart):
 class HorizontalDayChart(Chart):
     """Pretty much a horizontal bar chart, except for values it expects tuple
     of start and end time, and the whole thing hangs in air"""
+    def __init__(self, *args, **kwargs):
+        Chart.__init__(self, *args, **kwargs)
+        self.start_time, self.end_time = None, None
+    
     def plot_day(self, keys, data, start_time = None, end_time = None):
         self.keys, self.data = keys, data
         self.start_time, self.end_time = start_time, end_time
         self.show()
         self.redraw_canvas()
     
-    def _render(self):
+    def on_expose(self):
         context = self.context
-        Chart._render(self)
+        
+        Chart.on_expose(self)
         rowcount, keys = len(self.keys), self.keys
         
         start_hour = 0
@@ -611,43 +678,39 @@ class HorizontalDayChart(Chart):
             return
 
         
-        bar_width = int(self.graph_height / float(rowcount))
-        bar_width = min(bar_width, self.max_bar_width)
+        positions = {}
+        y = 0
+        bar_width = min(self.graph_height / float(len(self.keys)), self.max_bar_width)
+        for i, key in enumerate(self.keys):
+            positions[key] = (y + self.graph_y, round(bar_width - 1))
+            
+            y = y + round(bar_width)
+            bar_width = min(self.max_bar_width,
+                            (self.graph_height - y) / float(max(1, len(self.keys) - i - 1)))
+
 
         
         max_bar_size = self.graph_width - 15
-        gap = bar_width * 0.05
-
 
         self.layout.set_alignment(pango.ALIGN_RIGHT)
         self.layout.set_ellipsize(pango.ELLIPSIZE_END)
         
-        context.set_line_width(0)
-
         # bars and labels
         self.layout.set_width(legend_width * pango.SCALE)
 
         factor = max_bar_size / float(end_hour - start_hour)
 
-
         for i, label in enumerate(keys):
             self.set_color(graphics.Colors.aluminium[5])        
             
             self.layout.set_text(label)
             label_w, label_h = self.layout.get_pixel_size()
 
-            context.move_to(0, (bar_width * i) + (bar_width - label_h) / 2)
+            context.move_to(0, positions[label][0] + (positions[label][1] - label_h) / 2)
             context.show_layout(self.layout)
 
             base_color = self.bar_base_color or [220, 220, 220]
 
-            gap = bar_width * 0.05
-
-            bar_y = round(self.graph_y + (bar_width * i) + gap)
-
-            
-            bar_height = round(bar_width - (gap * 2))
-            
             if isinstance(self.data[i], list) == False:
                 self.data[i] = [self.data[i]]
             
@@ -655,10 +718,10 @@ class HorizontalDayChart(Chart):
                 bar_x = round((row[0]- start_hour) * factor)
                 bar_size = round((row[1] - start_hour) * factor - bar_x)
                 
-                self.draw_bar(self.graph_x + bar_x,
-                              bar_y,
+                self.draw_bar(round(self.graph_x + bar_x),
+                              positions[label][0],
                               bar_size,
-                              bar_height,
+                              positions[label][1],
                               base_color)
 
         #white grid and scale values
@@ -667,8 +730,9 @@ class HorizontalDayChart(Chart):
         context.set_line_width(1)
 
         pace = ((end_hour - start_hour) / 3) / 60 * 60
+        last_position = positions[keys[-1]]
         for i in range(start_hour + 60, end_hour, pace):
-            x = (i - start_hour) * factor
+            x = round((i - start_hour) * factor)
             
             minutes = i % (24 * 60)
 
@@ -676,14 +740,70 @@ class HorizontalDayChart(Chart):
             label_w, label_h = self.layout.get_pixel_size()
 
             context.move_to(self.graph_x + x - label_w / 2,
-                            bar_y + bar_height + 4)
+                            last_position[0] + last_position[1] + 4)
             self.set_color(graphics.Colors.aluminium[4])
             context.show_layout(self.layout)
 
             
             self.set_color((255, 255, 255))
-            self.context.move_to(self.graph_x + x, self.graph_y)
-            self.context.line_to(self.graph_x + x, bar_y + bar_height)
+            self.context.move_to(round(self.graph_x + x) + 0.5, self.graph_y)
+            self.context.line_to(round(self.graph_x + x) + 0.5,
+                                 last_position[0] + last_position[1])
 
                 
         context.stroke()
+
+
+""" sample usage """
+class BasicWindow:
+    def __init__(self):
+        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
+        window.set_title("Hamster Charting")
+        window.set_size_request(400, 300)
+        window.connect("delete_event", lambda *args: gtk.main_quit())
+    
+        self.stacked = BarChart(background = "#fafafa",
+                                bar_base_color = (220, 220, 220),
+                                legend_width = 20,
+                                show_stack_labels = True)
+        
+        box = gtk.VBox()
+        box.pack_start(self.stacked)
+
+
+        self.series = ["One", "Two", "Three", "Four", "Five", "Six", "Seven"]
+        self.stacks = ["x", "y", "z", "a", "b", "c", "d"]
+        self.stack_colors = dict([(stack, None) for stack in self.stacks])
+
+        import random
+        self.data = [[random.randint(0, 10) for j in range(len(self.stacks))] for i in range(len(self.series))]
+        
+        color_buttons = gtk.HBox()
+        color_buttons.set_spacing(4)
+
+        for stack in self.stacks:
+            button = gtk.ColorButton()
+            button.connect("color-set", self.on_color_set, stack)
+            color_buttons.pack_start(button)
+
+        box.pack_start(color_buttons, False)
+        
+        window.add(box)
+        window.show_all()
+
+        self.plot()
+        
+        
+    def plot(self):
+        self.stacked.stack_key_colors = self.stack_colors
+        self.stacked.plot(self.series, self.data, self.stacks)
+
+
+    def on_color_set(self, button, stack_idx):
+        self.stack_colors[stack_idx] = button.get_color().to_string()
+        self.plot()
+
+
+if __name__ == "__main__":
+   example = BasicWindow()
+   gtk.main()
