@@ -46,7 +46,6 @@ DB_FILE = 'hamster.db'
 
 class Storage(storage.Storage):
     con = None # Connection will be created on demand
-
     def __setup(self):
         """
         Delayed setup so we don't do everything at the same time
@@ -54,9 +53,12 @@ class Storage(storage.Storage):
         if self.__setup.im_func.complete:
             return
 
+        self.__con = None
+        self.__cur = None
+
         from configuration import runtime, GconfStore
 
-        db_file = runtime.database_file
+        db_file = runtime.database_path
         db_path, _ = os.path.split(os.path.realpath(db_file))
 
         if not os.path.exists(db_path):
@@ -97,7 +99,7 @@ class Storage(storage.Storage):
         """look up tags by their name. create if not found"""
         
         # filter descriptions out, just in case they have wandered in here
-        tags = [tag for tag in tags if tag.startswith("!") == False and len(tag.split(" ")) < 3]
+        tags = [tag.lower() for tag in tags if tag.startswith("!") == False and len(tag.split(" ")) < 3]
         
         db_tags = self.fetchall("select * from tags where name in (%s)"
                                             % ",".join(["?"] * len(tags)), tags) # bit of magic here - using sqlites bind variables
@@ -292,13 +294,14 @@ class Storage(storage.Storage):
                           a.description as description,
                           b.name AS name, b.id as activity_id,
                           coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id,
-                          e.name as tags
+                          e.name as tag
                      FROM facts a
                 LEFT JOIN activities b ON a.activity_id = b.id
                 LEFT JOIN categories c ON b.category_id = c.id
                 LEFT JOIN fact_tags d ON d.fact_id = a.id
                 LEFT JOIN tags e ON e.id = d.tag_id
-                    WHERE a.id = ? 
+                    WHERE a.id = ?
+                 ORDER BY e.name
         """
 
         return self.__group_tags(self.fetchall(query, (_("Unsorted"), id)))[0]
@@ -317,45 +320,25 @@ class Storage(storage.Storage):
             # we need dict so we can modify it (sqlite.Row is read only)
             # in python 2.5, sqlite does not have keys() yet, so we hardcode them (yay!)
             keys = ["id", "start_time", "end_time", "description", "name",
-                    "activity_id", "category", "category_id", "tags"]
+                    "activity_id", "category", "tag"]
             grouped_fact = dict([(key, grouped_fact[key]) for key in keys])
             
-            grouped_fact["tags"] = [ft["tags"] for ft in fact_tags if ft["tags"]]
+            grouped_fact["tags"] = [ft["tag"] for ft in fact_tags if ft["tag"]]
             grouped_facts.append(grouped_fact)
         return grouped_facts
 
     def __get_last_activity(self):
-        query = """
-                   SELECT a.id AS id,
-                          a.start_time AS start_time,
-                          a.end_time AS end_time,
-                          a.description as description,
-                          b.name AS name, b.id as activity_id,
-                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id,
-                          e.name as tags
-                     FROM facts a
-                LEFT JOIN activities b ON a.activity_id = b.id
-                LEFT JOIN categories c ON b.category_id = c.id
-                LEFT JOIN fact_tags d ON d.fact_id = a.id
-                LEFT JOIN tags e ON e.id = d.tag_id
-                    WHERE date(a.start_time) = ?
-                 ORDER BY a.start_time desc
-                    LIMIT 20
-        """
-        last = self.fetchall(query, (_("Unsorted"), dt.date.today()))        
-        if not last:
-            #try yesterday if there is nothing today
-            last = self.fetchall(query,
-                                 (# unsorted category
-                                  _("Unsorted"),
-                                  dt.date.today() - dt.timedelta(days=1)))
-        if last:
-            last = self.__group_tags(last)[0]
+        from configuration import GconfStore
+        day_start = GconfStore().get_day_start()
 
-        if last and last["end_time"]: #will consider as last only if it is going on
-           last = None
+        today = (dt.datetime.now() - dt.timedelta(hours = day_start.hour,
+                                                  minutes = day_start.minute)).date()
+        facts = self.__get_facts(today)
         
-        return last
+        last_activity = None
+        if facts and facts[-1]["end_time"] == None:
+            last_activity = facts[-1]
+        return last_activity
 
     def __touch_fact(self, fact, end_time):
         # tasks under one minute do not count
@@ -390,7 +373,7 @@ class Storage(storage.Storage):
         fact = self.fetchone(query, (start_time,
                                      start_time,
                                      start_time,
-                                     start_time + dt.timedelta(days=2)))
+                                     start_time + dt.timedelta(seconds = 60 * 60 * 12)))
 
         end_time = None        
 
@@ -523,29 +506,31 @@ class Storage(storage.Storage):
 
 
         # if we are working on +/- current day - check the last_activity
+        
         if (dt.datetime.now() - start_time <= dt.timedelta(days=1)):
-            last_activity = self.__get_last_activity()
+            previous = self.__get_last_activity()
 
-            if last_activity and last_activity['start_time'] < start_time:
-                #if this is the same, ongoing activity, then there is no need to create another one
-                if not tags and not activity.description \
-                   and last_activity['activity_id'] == activity_id:
-                    return last_activity
+            if previous and previous['start_time'] < start_time:
+                # check if maybe that is the same one, in that case no need to restart
+                if previous["activity_id"] == activity_id \
+                   and previous["tags"] == sorted([tag["name"] for tag in tags]) \
+                   and previous["description"] == (description or ""):
+                    return previous
                 
-                #if duration is less than a minute - it must have been a mistake
-                if not tags and not activity.description \
-                   and not last_activity["description"] \
-                   and 60 >= (start_time - last_activity['start_time']).seconds >= 0:
-                    self.__remove_fact(last_activity['id'])
-                    start_time = last_activity['start_time']
+                # otherwise, if not tags nor description is added (extra data
+                # see if maybe it is too short to qualify as an activity
+                if not previous["tags"] and not previous["description"] \
+                    and 60 >= (start_time - previous['start_time']).seconds >= 0:
+                    self.__remove_fact(previous['id'])
+                    start_time = previous['start_time']
                 else:
-                    #otherwise stop 
+                    # otherwise stop 
                     update = """
                                UPDATE facts
                                   SET end_time = ?
                                 WHERE id = ?
                     """
-                    self.execute(update, (start_time, last_activity["id"]))
+                    self.execute(update, (start_time, previous["id"]))
 
 
         # done with the current activity, now we can solve overlaps
@@ -572,15 +557,15 @@ class Storage(storage.Storage):
         return self.__get_fact(fact_id)
 
 
-    def __get_facts(self, date, end_date = None, category_id = None):
+    def __get_facts(self, date, end_date = None, search_terms = ""):
         query = """
                    SELECT a.id AS id,
                           a.start_time AS start_time,
                           a.end_time AS end_time,
                           a.description as description,
                           b.name AS name, b.id as activity_id,
-                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id,
-                          e.name as tags
+                          coalesce(c.name, ?) as category,
+                          e.name as tag
                      FROM facts a
                 LEFT JOIN activities b ON a.activity_id = b.id
                 LEFT JOIN categories c ON b.category_id = c.id
@@ -588,13 +573,46 @@ class Storage(storage.Storage):
                 LEFT JOIN tags e ON e.id = d.tag_id
                     WHERE (a.end_time >= ? OR a.end_time IS NULL) AND a.start_time <= ?
         """
-        
-        if category_id and isinstance(category_id, int):
-            query += " and b.category_id = %d" % category_id
-        elif category_id and isinstance(category_id, list):
-            query += " and b.category_id IN (%s)" % (",".join([str(id) for id in category_id]))
 
-        query += " ORDER BY a.start_time"
+        # let's see what we can do with search terms
+        # we will be looking in activity names, descriptions, categories and tags
+        # comma will be treated as OR
+        # space will be treated as AND or possible join
+        
+
+        # split by comma and then by space and remove all extra spaces
+        or_bits = [[term.strip().lower().replace("'", "''") #striping removing case sensitivity and escaping quotes in term
+                          for term in terms.strip().split(" ") if term.strip()]
+                          for terms in search_terms.split(",") if terms.strip()]
+
+        def all_fields(term):
+            return """(lower(a.description) like '%%%(term)s%%'
+                       or lower(b.name) = '%(term)s'
+                       or lower(c.name) = '%(term)s'
+                       or lower(e.name) = '%(term)s' )""" % dict(term = term)
+        
+        if or_bits:
+            search_query = "1<>1 " # will be building OR chain, so start with a false
+            
+            for and_bits in or_bits:
+                if len(and_bits) == 1:
+                    and_query = all_fields(and_bits[0])
+                else:
+                    and_query = "1=1 "  # will be building AND chain, so start with a true
+                    # if we have more than one word, go for "(a and b) or ab"
+                    # to match two word tags
+                    for bit1, bit2 in zip(and_bits, and_bits[1:]):
+                        and_query += "and (%s and %s) or %s" % (all_fields(bit1),
+                                                                all_fields(bit2),
+                                                                all_fields("%s %s" % (bit1, bit2)))
+                
+                search_query = "%s or (%s) " % (search_query, and_query)
+    
+            query = "%s and (%s)" % (query, search_query)
+        
+
+        
+        query += " ORDER BY a.start_time, e.name"
         end_date = end_date or date
 
         from configuration import GconfStore
@@ -604,7 +622,9 @@ class Storage(storage.Storage):
         datetime_from = dt.datetime.combine(date, split_time)
         datetime_to = dt.datetime.combine(end_date, split_time) + dt.timedelta(days = 1)
         
-        facts = self.fetchall(query, (_("Unsorted"), datetime_from, datetime_to))
+        facts = self.fetchall(query, (_("Unsorted"),
+                                      datetime_from,
+                                      datetime_to))
         
         #first let's put all tags in an array
         facts = self.__group_tags(facts)
@@ -620,7 +640,7 @@ class Storage(storage.Storage):
             elif (dt.date.today() - fact["start_time"].date()) <= dt.timedelta(days=1):
                 fact_end_time = dt.datetime.now().replace(microsecond = 0)
             else:
-                fact_end_time = fact["start_time"].replace(hour=23, minute=59)
+                fact_end_time = fact["start_time"]
 
             fact_start_date = fact["start_time"].date() \
                 - dt.timedelta(1 if fact["start_time"].time() < split_time else 0)
@@ -702,18 +722,6 @@ class Storage(storage.Storage):
             
         return activities
 
-    def __get_sorted_activities(self):
-        """returns list of acitivities that have categories"""
-        query = """
-                   SELECT a.*, b.name as category, b.category_order
-                     FROM activities a
-                LEFT JOIN categories b on coalesce(b.id, -1) = a.category_id
-                    WHERE a.category_id > -1
-                      AND a.deleted is null
-                 ORDER BY category_order, activity_order
-        """
-        return self.fetchall(query)
-        
     def __get_autocomplete_activities(self):
         """returns list of activities for autocomplete,
            activity names converted to lowercase"""
@@ -789,7 +797,7 @@ class Storage(storage.Storage):
     def get_connection(self):
         from configuration import runtime
         if self.con is None:
-            db_file = runtime.database_file
+            db_file = runtime.database_path
             self.con = sqlite.connect(db_file, detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
             self.con.row_factory = sqlite.Row
 
@@ -831,8 +839,8 @@ class Storage(storage.Storage):
         from configuration import runtime
         self.__setup()
 
-        con = self.connection
-        cur = con.cursor()
+        con = self.__con or self.connection
+        cur = self.__cur or con.cursor()
         
         if isinstance(statement, list) == False: #we kind of think that we will get list of instructions
             statement = [statement]
@@ -844,10 +852,27 @@ class Storage(storage.Storage):
          
                 res = cur.execute(statement[i], params[i])
 
-        con.commit()
-        cur.close()
+        if not self.__con:
+            con.commit()
+            cur.close()
+            runtime.register_modification()
+        
+        
+    def start_transaction(self):
+        # will give some hints to execute not to close or commit anything
+        self.__con = self.connection
+        self.__cur = self.__con.cursor()
+    
+    def end_transaction(self):
+        self.__con.commit()
+        self.__cur.close()
+        self.__con = None
+        from configuration import runtime
+        runtime.register_modification()
         
     def run_fixtures(self):
+        self.start_transaction()
+        
         # defaults
         work_category = {"name": _("Work"),
                          "entries": [_("Reading news"),
@@ -1061,3 +1086,5 @@ class Storage(storage.Storage):
             for entry in nonwork_category["entries"]:
                 self.__add_activity(entry, nonwork_cat_id)
 
+
+        self.end_transaction()
